"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _cloudfront = _interopRequireDefault(require("aws-sdk/clients/cloudfront"));

var _lodash = require("lodash");

var _createS3Bucket = _interopRequireDefault(require("./createS3Bucket"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(source, true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(source).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

const getDistributionId = arn => {
  const pattern = /.*\/(.*)$/;
  const match = pattern.exec(arn);
  return match[1];
};
/* eslint-disable no-console */


const getCloudFrontOriginAccessIdentity = async ({
  accessKeyId,
  secretAccessKey,
  region,
  Comment,
  Marker,
  MaxItems = '200'
}) => {
  console.log('getCloudFrontOriginAccessIdentity');
  const cloudFront = new _cloudfront.default({
    credentials: {
      accessKeyId,
      secretAccessKey
    },
    region
  });
  const {
    CloudFrontOriginAccessIdentityList
  } = await cloudFront.listCloudFrontOriginAccessIdentities({
    Marker,
    MaxItems
  }).promise();
  const {
    IsTruncated,
    NextMarker,
    Items
  } = CloudFrontOriginAccessIdentityList;
  const identity = Items.find(item => item.Comment === Comment);

  if (identity != null) {
    return {
      id: identity.Id,
      s3UserId: identity.S3CanonicalUserId
    };
  }

  if (IsTruncated) {
    return getCloudFrontOriginAccessIdentity({
      accessKeyId,
      secretAccessKey,
      region,
      Comment,
      Marker: NextMarker
    });
  }

  return null;
};

const ensureOriginAccessIdentity = async ({
  accessKeyId,
  secretAccessKey,
  region,
  stage
}) => {
  console.log('ensureOriginAccessIdentity');
  const cloudFront = new _cloudfront.default({
    credentials: {
      accessKeyId,
      secretAccessKey
    },
    region
  });
  const comment = `resolve-uploader-${stage}`;
  let identity = await getCloudFrontOriginAccessIdentity({
    accessKeyId,
    secretAccessKey,
    region,
    Comment: comment
  });

  if (!identity) {
    const {
      CloudFrontOriginAccessIdentity: {
        Id,
        S3CanonicalUserId
      }
    } = await cloudFront.createCloudFrontOriginAccessIdentity({
      CloudFrontOriginAccessIdentityConfig: {
        CallerReference: comment,
        Comment: comment
      }
    }).promise();
    identity = {
      id: Id,
      s3UserId: S3CanonicalUserId
    };
  }

  return identity;
};

const getCloudFrontDistributionArnByTags = async (cf, tagFilter, marker = null) => {
  console.log('getCloudFrontDistributionArnByTags');
  const {
    DistributionList: {
      NextMarker,
      Items,
      IsTruncated
    }
  } = await cf.listDistributions(marker ? {
    Marker: marker
  } : {}).promise();
  const taggedResources = [];

  for (const {
    ARN: arn
  } of Items) {
    const {
      Tags: {
        Items: tags
      }
    } = await cf.listTagsForResource({
      Resource: arn
    }).promise();
    taggedResources.push({
      arn,
      tags
    });
  }

  const resource = taggedResources.find(({
    tags
  }) => (0, _lodash.differenceWith)(tagFilter, tags, _lodash.isEqual).length === 0);

  if (resource) {
    return resource.arn;
  }

  if (IsTruncated) {
    return getCloudFrontDistributionArnByTags(cf, tagFilter, NextMarker);
  }

  return null;
};

const ensureDistribution = async ({
  accessKeyId,
  secretAccessKey,
  region,
  bucketName,
  domainName,
  domainCertificateArn,
  callerReference,
  originAccessIdentity,
  functionVersionArn,
  tags
}) => {
  console.log('ensureDistribution');
  const originId = `s3-${bucketName}`;
  const originDomain = `${bucketName}.s3.amazonaws.com`;
  const lambdaAssociations = [{
    LambdaFunctionARN: functionVersionArn,
    EventType: 'origin-request',
    IncludeBody: true
  }];
  const forwardHeaders = ['Access-Control-Request-Method', 'Access-Control-Request-Headers', 'Accept-Encoding', 'Accept-Language', 'Accept', 'Authorization', 'Content-Type', 'Host', 'Origin', 'User-Agent'];
  const cloudFront = new _cloudfront.default({
    credentials: {
      accessKeyId,
      secretAccessKey
    },
    region
  });
  const distributionConfig = {
    CallerReference: callerReference,
    Comment: callerReference,
    Enabled: true,
    Aliases: {
      Quantity: 1,
      Items: [`uploader.${domainName}`]
    },
    Origins: {
      Quantity: 1,
      Items: [{
        Id: originId,
        DomainName: originDomain,
        OriginPath: '',
        S3OriginConfig: {
          OriginAccessIdentity: `origin-access-identity/cloudfront/${originAccessIdentity.id}`
        },
        CustomHeaders: {
          Quantity: 0,
          Items: []
        }
      }]
    },
    Restrictions: {
      GeoRestriction: {
        RestrictionType: 'none',
        Quantity: 0,
        Items: []
      }
    },
    WebACLId: '',
    HttpVersion: 'http2',
    IsIPV6Enabled: true,
    ViewerCertificate: {
      CloudFrontDefaultCertificate: false,
      ACMCertificateArn: domainCertificateArn,
      SSLSupportMethod: 'sni-only',
      // eslint-disable-next-line spellcheck/spell-checker
      MinimumProtocolVersion: 'TLSv1.1_2016'
    },
    DefaultCacheBehavior: {
      TargetOriginId: originId,
      ViewerProtocolPolicy: 'redirect-to-https',
      SmoothStreaming: false,
      MinTTL: 0,
      DefaultTTL: 300,
      MaxTTL: 31536000,
      Compress: true,
      FieldLevelEncryptionId: '',
      TrustedSigners: {
        Enabled: false,
        Quantity: 0
      },
      AllowedMethods: {
        Quantity: 3,
        Items: ['GET', 'HEAD', 'OPTIONS'],
        CachedMethods: {
          Quantity: 2,
          Items: ['GET', 'HEAD']
        }
      },
      ForwardedValues: {
        QueryString: true,
        QueryStringCacheKeys: {
          Quantity: 0,
          Items: []
        },
        Cookies: {
          Forward: 'none'
        },
        Headers: {
          Quantity: forwardHeaders.length,
          Items: forwardHeaders
        }
      },
      LambdaFunctionAssociations: {
        Quantity: lambdaAssociations.length,
        Items: lambdaAssociations
      }
    }
  };
  const cfDistributionArn = await getCloudFrontDistributionArnByTags(cloudFront, tags);
  let cfDomainName = null;

  if (cfDistributionArn == null) {
    const {
      DomainName
    } = await cloudFront.createDistributionWithTags({
      DistributionConfigWithTags: {
        DistributionConfig: distributionConfig,
        Tags: {
          Items: tags
        }
      }
    }).promise();
    cfDomainName = DomainName;
  } else {
    const distributionId = getDistributionId(cfDistributionArn);
    const {
      Distribution: {
        DistributionConfig: currentConfig
      },
      ETag
    } = await cloudFront.getDistribution({
      Id: distributionId
    }).promise();

    const updatedConfig = _objectSpread({}, currentConfig, {}, distributionConfig);

    const {
      Distribution: {
        DomainName
      }
    } = await cloudFront.updateDistribution({
      DistributionConfig: updatedConfig,
      Id: distributionId,
      IfMatch: ETag
    }).promise();
    cfDomainName = DomainName;
  }

  return cfDomainName;
};

const createCloudFrontDistribution = async ({
  accessKeyId,
  secretAccessKey,
  region,
  bucketName
}, {
  domainName,
  domainCertificateArn,
  callerReference,
  stageTagName,
  stage,
  functionVersionArn
}) => {
  console.log('createCloudFrontDistribution');
  const tags = [{
    Key: stageTagName,
    Value: stage
  }, {
    Key: 'resolve-type',
    Value: 'cloudfront-distribution-uploader'
  }];
  const originAccessIdentity = await ensureOriginAccessIdentity({
    accessKeyId,
    secretAccessKey,
    region,
    stage
  });
  await (0, _createS3Bucket.default)({
    accessKeyId,
    secretAccessKey,
    region,
    Bucket: bucketName,
    ACL: 'private',
    originAccessIdentity
  });
  return await ensureDistribution({
    accessKeyId,
    secretAccessKey,
    region,
    bucketName,
    domainName,
    domainCertificateArn,
    callerReference,
    originAccessIdentity,
    functionVersionArn,
    tags
  });
};
/* eslint-enable no-console */


var _default = createCloudFrontDistribution;
exports.default = _default;
//# sourceMappingURL=createCloudFrontDistribution.js.map